@@ -286,7 +286,7 @@ GetHostWithPort(char *token, Ip::Address *ipa)
 
     if (NULL == host)
         ipa->setAnyAddr();
-    else if ( ipa->GetHostByName(host) ) /* dont use ipcache. Accept either FQDN or IPA. */
+    else if (ipa->GetHostByName(host)) /* do not use ipcache. Accept either FQDN or IPA. */
         (void) 0;
     else
         return false;
